@@ -7,10 +7,6 @@
 # devtools::check(cleanup=FALSE) # check timings of examples: read log
 # devtools::check
 
-# Prevent sp from callin code in rgdal or rgeos, which are retiring
-# (see https://r-spatial.org/r/2022/04/12/evolution.html#packages-depending-on-sp-and-raster)
-# devtools::install_github("rsbivand/sp@evolution")
-# Sys.setenv("_SP_EVOLUTION_STATUS_"=2)
 
 #' @title  Package for analyzing DEPONS simulation output
 #' @name DEPONS2R
@@ -46,59 +42,35 @@ NULL
 #' @return Returns a \code{POSIXlt} object
 #' @seealso \code{\link{get.latest.sim}}
 #' @export get.simtime
-get.simtime <- function(fname=NULL, tz="GMT") {
-  # chg double-dots to _
-  fn <- gsub("..", "_", fname, fixed=TRUE)
+get.simtime <- function (fname = NULL, tz = "UTC") {
+  fn <- gsub("..", "_", fname, fixed = TRUE)
   ncf <- nchar(fn)
-  time.string <- substr(fn, ncf-23, ncf-4)
+  time.string <- substr(fn, ncf - 23, ncf - 4)
   time.string <- gsub("_", ":", time.string)
-  # Convert text months to numbers
-  template.months <- substr(100+(1:12), 2, 3)
+  template.months <- substr(100 + (1:12), 2, 3)
   template.dates <- paste0("2000-", template.months, "-01")
-  system.months <- months(as.POSIXlt(template.dates, tz="GMT"), abbreviate=TRUE)
-  time.string <- gsub("Jan", "01", time.string)
-  time.string <- gsub("jan", "01", time.string)
-  time.string <- gsub(system.months[1], "01", time.string)
-  time.string <- gsub("Feb", "02", time.string)
-  time.string <- gsub("feb", "02", time.string)
-  time.string <- gsub(system.months[2], "02", time.string)
-  time.string <- gsub("Mar", "03", time.string)
-  time.string <- gsub("mar", "03", time.string)
-  time.string <- gsub(system.months[3], "03", time.string)
-  time.string <- gsub("Apr", "04", time.string)
-  time.string <- gsub("apr", "04", time.string)
-  time.string <- gsub(system.months[4], "04", time.string)
-  time.string <- gsub("May", "05", time.string)
-  time.string <- gsub("maj", "05", time.string)
-  time.string <- gsub(system.months[5], "05", time.string)
-  time.string <- gsub("Jun", "06", time.string)
-  time.string <- gsub("jun", "06", time.string)
-  time.string <- gsub(system.months[6], "06", time.string)
-  time.string <- gsub("Jul", "07", time.string)
-  time.string <- gsub("jul", "07", time.string)
-  time.string <- gsub(system.months[7], "07", time.string)
-  time.string <- gsub("Aug", "08", time.string)
-  time.string <- gsub("aug", "08", time.string)
-  time.string <- gsub(system.months[8], "08", time.string)
-  time.string <- gsub("Sep", "09", time.string)
-  time.string <- gsub("sep", "09", time.string)
-  time.string <- gsub(system.months[9], "09", time.string)
-  time.string <- gsub("Oct", "10", time.string)
-  time.string <- gsub("okt", "10", time.string)
-  time.string <- gsub(system.months[10], "10", time.string)
-  time.string <- gsub("Nov", "11", time.string)
-  time.string <- gsub("nov", "11", time.string)
-  time.string <- gsub(system.months[11], "11", time.string)
-  time.string <- gsub("Dec", "12", time.string)
-  time.string <- gsub("dec", "12", time.string)
-  time.string <- gsub(system.months[12], "12", time.string)
+  system.months <- months(as.POSIXlt(template.dates, tz = "UTC"),
+                          abbreviate = TRUE)
+  time.string <- gsub(paste0("Jan|jan|", system.months[1]), "01", time.string)
+  time.string <- gsub(paste0("Feb|feb|", system.months[2]), "02", time.string)
+  time.string <- gsub(paste0("Mar|mar|", system.months[3]), "03", time.string)
+  time.string <- gsub(paste0("Apr|apr|", system.months[4]), "04", time.string)
+  time.string <- gsub(paste0("May|may|", system.months[5]), "05", time.string)
+  time.string <- gsub(paste0("Jun|jun|", system.months[6]), "06", time.string)
+  time.string <- gsub(paste0("Jul|jul|", system.months[7]), "07", time.string)
+  time.string <- gsub(paste0("Aug|aug|", system.months[8]), "08", time.string)
+  time.string <- gsub(paste0("Sep|sep|", system.months[9]), "09", time.string)
+  time.string <- gsub(paste0("Oct|oct|", system.months[10]), "10", time.string)
+  time.string <- gsub(paste0("Nov|nov|", system.months[11]), "11", time.string)
+  time.string <- gsub(paste0("Dec|dec|", system.months[12]), "12", time.string)
   substr(time.string, 5, 5) <- "-"
   substr(time.string, 8, 8) <- "-"
   substr(time.string, 11, 11) <- " "
-  time.posix <- as.POSIXlt(time.string, tz=tz)
-  if (any(inherits(time.posix,"POSIXlt")))
+  time.posix <- as.POSIXlt(time.string, tz = tz)
+  if (any(inherits(time.posix, "POSIXlt")))
     return(time.posix)
-  stop(paste0("Could not extract date correctly from ", fname), ". Got ", time.string)
+  stop(paste0("Could not extract date correctly from ", fname),
+       ". Got ", time.string)
 }
 
 
@@ -316,25 +288,22 @@ make.windfarms <- function(area.file, area.def, n.wf, n.turb, turb.dist,
 #' @param tick Numeric, or numeric vector; tick number
 #' @param timestep Numeric; length of each simulation time step, in minutes.
 #' Defaults to 30 minutes.
-#' @param origin Character. The first day in the period that the simulation represents,
+#' @param origin Character. The first day of the period that the simulation represents,
 #' format: 'yyyy-mm-dd'.
-#' @param ... Optional parameters, including time zone (tz)
+#' @param tz Character. Valid time zone code (default UTC).
+#' @param ... Optional parameters
 #' @note The function assumes that there are 30 days in each month, except in
 #' January, February and March with 31, 28 and 31 days, respectively.
 #' @return object of class \code{\link{as.POSIXlt}}
 #' @export tick.to.time
-tick.to.time <- function(tick, timestep=30, origin="2010-01-01", ...) {
+#' @seealso \code{\link{time.to.tick}} is the inverse of this function, converting dates to ticks
+tick.to.time <- function(tick, timestep=30, origin="2010-01-01", tz = "UTC", ...) {
   old <- options()
   on.exit(options(old))  # Reset user options on exit
   if(min(tick)<0) stop("tick should be positive and numeric")
   if(!is.numeric(timestep)) stop("'timestep' should be numeric")
   if(!is.character(origin)) stop("'origin' should be character, in the format 'yyyy-mm-dd'")
-  if(!hasArg(tz)) {
-    tz <- "GMT"
-  } else {
-    tz <- as.character(list(...)[["tz"]])
-    tz <- tz
-  }
+
   doy <- 1:360
   # Make months and days in sim year
   mm <- rep(NA, length(doy))
@@ -354,15 +323,16 @@ tick.to.time <- function(tick, timestep=30, origin="2010-01-01", ...) {
   start.d <- as.numeric(substr(origin, 9, 10))
   start.doy <- which(mm==start.m & dd==start.d)
   # Date that the sim is supposed to correspond to
-  sim.day <- trunc((tick-1)/((24*60)/timestep))+1
+  sim.day <- ceiling((tick + 1)/((24 * 60)/timestep))
   min.vector.lgt <- max(sim.day)+start.doy
   mm2 <- rep(mm, ceiling(min.vector.lgt/360))
   dd2 <- rep(dd, ceiling(min.vector.lgt/360))
   sim.month <- mm2[sim.day + start.doy - 1]
   sim.day.of.month <- dd2[sim.day + start.doy - 1]
-  sim.year <- start.y+trunc((sim.day  + start.doy - 1)/360)
+  sim.year <- start.y+trunc((sim.day  + start.doy - 2)/360)
   sim.date <- paste(sim.year, substr(sim.month+100, 2, 3), substr(sim.day.of.month+100, 2, 3), sep="-")
   sim.date <- as.POSIXlt(sim.date, tz=tz)
+  start.hms <- as.POSIXlt(origin)$hour * 3600 + as.POSIXlt(origin)$min * 60 + as.POSIXlt(origin)$sec
   sim.second.of.day <- (tick*timestep*60) %% (24*60*60)
   out <- sim.date+sim.second.of.day
   return(out)
@@ -371,6 +341,132 @@ tick.to.time <- function(tick, timestep=30, origin="2010-01-01", ...) {
 
 
 
+
+#'Convert date to tick number
+#'
+#'@description
+#'Convert a date to the number of ticks since simulation start while taking
+#'into account that DEPONS assumes that there are 360 days in a simulation year.
+#'
+#'@usage
+#'time.to.tick(time, timestep = 30, origin = "2010-01-01", tz = "UTC", ...)
+#'
+#'@details
+#'Times are rounded down to the current 30-minute interval during conversion.
+#'The function assumes that there are 30 days in each month, except in January,
+#'February and March with 31, 28 and 31 days, respectively. Provided dates that
+#'fall on days that are not accommodated (February 29, and the 31st day of the
+#'months May, July, August, October, and December) are returned as NA.
+#'
+#'The function may be used to, e.g., convert recorded piling dates to ticks for
+#'use in wind farm scenarios (see \code{\link{make.windfarms}} for construction
+#'of hypothetical scenarios from parametric inputs).
+#'
+#'@param time Character, or character vector, of the form 'YYYY-MM-DD'
+#'(or 'YYYY-MM-DD HH:MM:SS'), or equivalent POSIX object. Date(s) to be converted
+#'to ticks.
+#'@param timestep Numeric (default 30). Length of each simulation time step in
+#'minutes.
+#'@param origin Character of the form 'YYYY-MM-DD' or equivalent POSIX object
+#'(default "2010-01-01"). Start date of simulation.
+#'@param tz Character. Valid time zone code (default UTC).
+#'@param ... Optional parameters.
+#'
+#'@returns
+#'Numeric vector of tick numbers.
+#'
+#'@examples
+#'\dontrun{
+#'#Uses date column of AIS data.
+#'#Times are in 30-minute intervals, and converting back yields the same times
+#'data(aisdata)
+#'ticks <- time.to.tick(aisdata$time, origin = "2015-12-20")
+#'times_reconverted <- tick.to.time(ticks, origin = "2015-12-20")
+#'
+#'#Uses dates at other intervals.
+#'#Converting back yields times rounded down to the current 30-minute interval
+#'times <- c("2016-12-20 00:10:00",
+#'           "2016-12-20 02:45:30",
+#'           "2016-12-20 05:01:05",
+#'           "2016-12-22 01:30:00")
+#'ticks <- time.to.tick(times, origin = "2015-12-20")
+#'times_reconverted <- tick.to.time(ticks, origin = "2015-12-20")}
+#'
+#'@seealso \code{\link{tick.to.time}} is the inverse of this function, converting
+#'ticks to dates
+
+time.to.tick <- function (time, timestep = 30, origin = "2010-01-01", tz = "UTC", ...) {
+
+  old <- options()
+  on.exit(options(old))
+  origin <- try(as.POSIXlt(origin, tz = "UTC"))
+  if (!("POSIXlt" %in% class(origin))) {
+    stop("'origin' must be of the form 'YYYY-MM-DD' for conversion to POSIX")
+  }
+  if (!is.numeric(timestep))
+    stop("'timestep' should be numeric")
+
+  origin <- as.POSIXlt(origin, tz = tz)
+  monlng <- as.list(c(31,28,31, rep(30, times=9))) # list of expected month lengths
+  names(monlng) <- seq(1:12)-1
+
+  # function to step through months of appropriate length and add up ticks
+  count.ticks.from.origin <- function(origin, target) {
+    if (origin > target) stop ("origin is later than time")
+    cur.or <- origin
+    cur.count <- 0
+    if (cur.or$year < target$year || cur.or$mon < target$mon) { # at start, if not in target month & year yet, bottom up to next month to enable simple month-length additions in the following
+      new.or <- cur.or
+      new.or$mon <- new.or$mon + 1
+      new.or$mday <- 1
+      new.or$hour <- 0
+      new.or$min <- 0
+      new.or$sec <- 0
+      difference <- as.numeric(difftime(new.or, cur.or, units = "secs"))
+      cur.diff <- (monlng[[as.character(cur.or$mon)]] - (cur.or$mday - 1)) * (24 * 60 * 60) # account for skipped time, observing expected duration of MODEL month
+      cur.count <- cur.count + cur.diff
+      cur.or <- new.or
+    }
+    breaks <- 0
+    while (breaks == 0) { # then proceed through remaining months
+      cur.or <- as.POSIXlt(as.POSIXct(cur.or, tz = "UTC"), tz = "UTC") # reset format to roll over added months
+      if (cur.or > target) stop (paste0("tick accumulation overshot target date. Failed at ", time[instance]))
+      cur.diff <- as.numeric(difftime(target, cur.or, units = "secs"))
+      if (cur.or$year == target$year && cur.or$mon == target$mon) { # if reached same month and same year, add remainder secs to count
+        cur.count <- cur.count + cur.diff
+        breaks <- 1 # and exit
+      } else { # else add one months' (of correct length) worth of seconds to moving origin and count
+        cur.count <- cur.count + monlng[[as.character(cur.or$mon)]] * 24 * 60 * 60 # but MODEL month's worth of seconds to count
+        cur.or$mon <- cur.or$mon + 1
+      } # and repeat
+    }
+    ticks <- floor(cur.count / (60 * timestep))
+    return (ticks)
+  }
+
+  time.converted <- time
+  for (instance in 1:length(time)) {
+    test <- try(as.POSIXlt(time[instance], tz = tz))
+    if (!("POSIXlt" %in% class(test))) {
+      stop(paste0("'time' values must be of the form 'YYYY-MM-DD' (optionally 'YYYY-MM-DD HH:MM:SS') for conversion to POSIX. Failed at ", time[instance]))
+    }
+    target <- as.POSIXlt(time[instance], tz = tz)
+    if (target$mday > monlng[[target$mon + 1]]) { # if a date (start or end) is on a day beyond expected month length, set instance to NA and continue to next instance
+      time.converted[instance] <- NA
+      message(paste0("time set to NA due to date outside accommodated month length: ", time[instance]))
+      next
+    } else {
+      time.converted[instance] <- count.ticks.from.origin (origin, target) # else process ticks for date
+    }
+  }
+
+  return(as.numeric(time.converted))
+}
+
+
+
+
+
 setGeneric("make.clip.poly", function(bbox, ...){})
 
 #' @name make.clip.poly
